/* SPDX-License-Identifier: GPL-2.0
--------------------------------------------------------------------------------
--
--	syn1588(R) NIC
--
--------------------------------------------------------------------------------
--
--	Oregano Systems - Design & Consulting GesmbH
--
--	Web: http://oregano.at
@: support@oregano.at
--
--------------------------------------------------------------------------------
--
--	Copyright (c) 2006-2011
--
--------------------------------------------------------------------------------
--
--	Confidential data.
--
-- This source code, netlists, documentation is confidential and must not be
-- published nor made available to third parties. Usage of this source code,
-- netlists, documentation is subject to restrictions and conditions of a
-- license agreement.
--
-- ALL RIGHTS RESERVED.
-- Oregano Systems does not assume any liability
-- arising out of the application or use of any product described or shown
-- herein nor does it convey any license under its patents, copyrights, or any
-- rights of others.
--
-- Oregano Systems reserves the right to make changes, at any time, in
-- order to improve reliability, function or design. Oregano Systems will not
-- assume responsibility for the use of any circuitry described herein.
--
-- All trademarks used are the property of their respective owners.
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-- MA	02110-1301, USA.
--
--------------------------------------------------------------------------------
--
--	Author:	 Patrick Loschmidt
--	Austrian Academy of Science
--	Institute for Integrated Sensor Systems
--	Filename:	 SyncDriver.h
--	Date of Creation: Fri 2006-06-16
--
--	Description: the synchronisation driver char module header file
--	Requirements: syn1588nic driver, Linux kernel version >=2.6.16
--
-------------------------------------------------------------------------------
*/

#ifndef _CHAR_SYNCD_H_
#define _CHAR_SYNCD_H_
#include "clock_m.h"
#include "syn1588_dev.h"

#include <linux/atomic.h>
#include <linux/miscdevice.h>
#include <linux/poll.h>

#define WR_VERIFY 1 ///< enable device driver write access verify


struct clock_m_device;
struct nic_priv_data;

struct oregano_syncd_platform_data {
	struct syn1588_dev *syn1588_dev;
};

/**
 * \brief synchronization device private data structure
 * stores device information (per process)
 */
struct syncD_device {
	struct device *dev;
	char name[10];

	struct miscdevice miscdev; ///< pointer to misc device structure
	int dev_nr; ///< misc device number

	struct rw_semaphore
		virtdev_list_sem; ///< mutual exclusion semaphore (blocks
		///changes during access)
	spinlock_t
		virtdev_lock; ///< locking between lineEvent and write process (open, close)
	/* virtual device list */
	struct list_head virtdev_list; ///< list of open virtual devices
	atomic_t virtdev_count; ///< number of open virtual devices
	struct fasync_struct *async_queue; ///< list of asynchronous readers

	struct syn1588_dev syn1588_dev;

	struct oregano_clock_callback clock_m_callback;

	struct device *updater_dev; // Nullable



	// deprecated
	void __iomem *iobase_spi; // Nullable
	void __iomem *iobase_i2c_video; // Nullable
	void __iomem *iobase_eeprom; // Nullable

	spinlock_t virtreg_lock; ///< used to lock virtual registers
	// enum {off, prepare, wait, amort, finished} sAmortState;
	// indicates status of synchronous amortisation
	u32 eventctrl; ///< virtual event control register
	u32 timectrl; ///< virtual time control register
};

/**
 * \brief virtual device structure
 * stores device information (per file descriptor)
 */
struct syncD_virtdev {
	struct list_head list;
	struct syncD_device *syncd_dev; ///< pointer to real clock_m device structure
	pid_t pid; ///< used to identify generating process
	struct file *access_key; ///< used to identify each open command
	wait_queue_head_t irq_waitq; ///< wait queue for interrupts (blocking I/O)

	DECLARE_BITMAP(used_period_ids, __SYN1588_IOCTL_PERIOD_NBITS); // bitmap from enum syn1588_ioctl_period_id
	int event_id; // referenced event -1 disabled
	DECLARE_BITMAP(used_trigger_ids, __SYN1588_IOCTL_TRIGGER_NBITS); // bitmap from enum syn1588_ioctl_trigger_id
	DECLARE_BITMAP(used_io_port_ids, __SYN1588_IOCTL_IO_PORT_NBITS); // bitmap from enum syn1588_ioctl_io_port_id

	// ---- Deprecated ----
	// virtual interrupt (per process)
	int irq_fail; /// <indicates multiple interrupts without readout
	u32 irsrc; ///< virtual interrupt source register
	u32 iren; ///< virtual interrupt enable register
	u32 time_m, time_h, time_e; ///< virtual time register (low is the HW)
	u32 time2_h, time2_e;
	u32 systs_l, systs_h, hwts_l,
		hwts_h; ///< variables for LSync helper registers
	u32 delta_sys, delta_hw;
	struct rw_semaphore
		access_sem; ///< mutual exclusion semaphore (blocks removing during access)

	//irq_handler[32]
};

/**
 * \name Prototypes for internal functions
 *@{
 */
int syncD_fasync(int fd, struct file *filp, int mode);
int syncD_dev_register(struct clock_m_device *clock_m);
void syncD_dev_unregister(struct clock_m_device *clock_m);
int syncD_dev_init_loadmodul(void);
void syncD_dev_exit_unloadmodul(void);
/*@}*/

#endif /* _CHAR_SYNCD_H_ */
