/* SPDX-License-Identifier: GPL-2.0
 *
 * Copyright (C) 2020 Norbert Summer <summer@oregano.at>. All Rights Reserved.
 */

// this file garanties compatibilty between diferen kernel versions

#ifndef _SYN1588_COMPATIBILITY_H
#define _SYN1588_COMPATIBILITY_H

#include "compat_math.h"
#include "compat_rhel.h"
#include "time.h"
#include "compat_math.h"

#include <linux/device.h>
#include <linux/version.h>


#if defined(CONFIG_SUSE_KERNEL)
#include <linux/suse_version.h>

#if SUSE_VERSION >= 15 && SUSE_PATCHLEVEL >= 2 && \
	SUSE_VERSION < 16
#define SUSE15SP2_BACKPORT_NDO_TX_TIMEOUT
#define SUSE15SP2_BACKPORT_DEVM_IOREMAP
#endif

#if SUSE_VERSION >= 15 && SUSE_PATCHLEVEL >= 3 && \
	SUSE_VERSION < 16
#define SUSE15SP3_PHY_DO_IOCTRL
#endif
#endif // end if CONFIG_SUSE_KERNEL


#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 32)
#error "SYN1588 requires Linux >= 2.6.32, please contact oregano support if you need support for this version"
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 5, 0)
#warning "SYN1588 is not tested for version >6.4.x"
#endif

#include "time.h"


#if KERNEL_VERSION(6, 3, 0) > LINUX_VERSION_CODE
#define mdiobus_scan_c22	mdiobus_scan
#else
#define mdiobus_scan_c22	mdiobus_scan_c22
#endif


#if  KERNEL_VERSION(5, 0, 0) <= LINUX_VERSION_CODE || defined(RHEL8_BACKPORT_LINKMODE_TEST_BIT)

#include <linux/linkmode.h>
#define linkmode_test_bit_compat linkmode_test_bit
#define linkmode_mod_bit_compat linkmode_mod_bit

#elif (KERNEL_VERSION(4, 20, 0) <= LINUX_VERSION_CODE)
static inline bool linkmode_test_bit_compat(int nr, const volatile unsigned long *addr)
{
	return test_bit(nr, addr);
}
static inline void linkmode_mod_bit_compat(int nr, volatile unsigned long *addr, int set)
{
	if (set)
		set_bit(nr, addr);
	else
		clear_bit(nr, addr);
}
static inline void linkmode_set_bit(int nr, volatile unsigned long *addr) {
	linkmode_mod_bit_compat(nr, addr, true);
}
static inline void linkmode_copy( unsigned long *dest, const unsigned long *src)
{
	bitmap_copy(dest, src, __ETHTOOL_LINK_MODE_MASK_NBITS);
}
#else // older than 4.20
static inline bool linkmode_test_bit_compat(int nr, unsigned long value) {
	if(nr >=32) {
		return false;
	}
	return test_bit(nr, &value);
}
// hack to get a ref to the bitset value
#define linkmode_mod_bit_compat(nr, value, set) _linkmode_mod_bit_compat(nr, (unsigned long *)&value, set)
static inline void _linkmode_mod_bit_compat(int nr, unsigned long *value, int set)
{
	if(nr >=32) {
		return;
	}
	if (set)
		set_bit(nr, value);
	else
		clear_bit(nr, value);
}
#define linkmode_copy(dest, src) _linkmode_copy((unsigned long *)&dest, src)
static inline void _linkmode_copy( unsigned long *dest, const unsigned long src)
{
	*dest = src;
}
#define linkmode_set_bit(nr, addr) _linkmode_set_bit(nr, (unsigned long *)&addr)
static inline void _linkmode_set_bit(int nr, unsigned long *addr) {
	linkmode_mod_bit_compat(nr, addr, true);
}
#endif

#if  KERNEL_VERSION(5, 2, 0) > LINUX_VERSION_CODE || defined(RHEL8_BACKPORT_LINKMODE_TEST_BIT)

#if (KERNEL_VERSION(4, 20, 0) <= LINUX_VERSION_CODE)
#define phy_copy_pause_bits _phy_copy_pause_bits
#else
#define phy_copy_pause_bits(dst, src) _phy_copy_pause_bits((unsigned long *)&dst, (unsigned long *)&src )
#endif

#include <linux/ethtool.h>
//dynamic detect backport of `enum ethtool_link_mode_bit_indices`
#ifndef __ETHTOOL_LINK_MODE_LEGACY_MASK
/* Link mode bit indices */
enum ethtool_link_mode_bit_indices {
	ETHTOOL_LINK_MODE_10baseT_Half_BIT	= 0,
	ETHTOOL_LINK_MODE_10baseT_Full_BIT	= 1,
	ETHTOOL_LINK_MODE_100baseT_Half_BIT	= 2,
	ETHTOOL_LINK_MODE_100baseT_Full_BIT	= 3,
	ETHTOOL_LINK_MODE_1000baseT_Half_BIT	= 4,
	ETHTOOL_LINK_MODE_1000baseT_Full_BIT	= 5,
	ETHTOOL_LINK_MODE_Autoneg_BIT		= 6,
	ETHTOOL_LINK_MODE_TP_BIT		= 7,
	ETHTOOL_LINK_MODE_AUI_BIT		= 8,
	ETHTOOL_LINK_MODE_MII_BIT		= 9,
	ETHTOOL_LINK_MODE_FIBRE_BIT		= 10,
	ETHTOOL_LINK_MODE_BNC_BIT		= 11,
	ETHTOOL_LINK_MODE_10000baseT_Full_BIT	= 12,
	ETHTOOL_LINK_MODE_Pause_BIT		= 13,
	ETHTOOL_LINK_MODE_Asym_Pause_BIT	= 14,
	ETHTOOL_LINK_MODE_2500baseX_Full_BIT	= 15,
	ETHTOOL_LINK_MODE_Backplane_BIT		= 16,
	ETHTOOL_LINK_MODE_1000baseKX_Full_BIT	= 17,
	ETHTOOL_LINK_MODE_10000baseKX4_Full_BIT	= 18,
	ETHTOOL_LINK_MODE_10000baseKR_Full_BIT	= 19,
	ETHTOOL_LINK_MODE_10000baseR_FEC_BIT	= 20,
	ETHTOOL_LINK_MODE_20000baseMLD2_Full_BIT = 21,
	ETHTOOL_LINK_MODE_20000baseKR2_Full_BIT	= 22,
	ETHTOOL_LINK_MODE_40000baseKR4_Full_BIT	= 23,
	ETHTOOL_LINK_MODE_40000baseCR4_Full_BIT	= 24,
	ETHTOOL_LINK_MODE_40000baseSR4_Full_BIT	= 25,
	ETHTOOL_LINK_MODE_40000baseLR4_Full_BIT	= 26,
	ETHTOOL_LINK_MODE_56000baseKR4_Full_BIT	= 27,
	ETHTOOL_LINK_MODE_56000baseCR4_Full_BIT	= 28,
	ETHTOOL_LINK_MODE_56000baseSR4_Full_BIT	= 29,
	ETHTOOL_LINK_MODE_56000baseLR4_Full_BIT	= 30,
	ETHTOOL_LINK_MODE_25000baseCR_Full_BIT	= 31,

	/* Last allowed bit for __ETHTOOL_LINK_MODE_LEGACY_MASK is bit
	 * 31. Please do NOT define any SUPPORTED_* or ADVERTISED_*
	 * macro for bits > 31. The only way to use indices > 31 is to
	 * use the new ETHTOOL_GLINKSETTINGS/ETHTOOL_SLINKSETTINGS API.
	 */

	ETHTOOL_LINK_MODE_25000baseKR_Full_BIT	= 32,
	ETHTOOL_LINK_MODE_25000baseSR_Full_BIT	= 33,
	ETHTOOL_LINK_MODE_50000baseCR2_Full_BIT	= 34,
	ETHTOOL_LINK_MODE_50000baseKR2_Full_BIT	= 35,
	ETHTOOL_LINK_MODE_100000baseKR4_Full_BIT	= 36,
	ETHTOOL_LINK_MODE_100000baseSR4_Full_BIT	= 37,
	ETHTOOL_LINK_MODE_100000baseCR4_Full_BIT	= 38,
	ETHTOOL_LINK_MODE_100000baseLR4_ER4_Full_BIT	= 39,
	ETHTOOL_LINK_MODE_50000baseSR2_Full_BIT		= 40,
	ETHTOOL_LINK_MODE_1000baseX_Full_BIT	= 41,
	ETHTOOL_LINK_MODE_10000baseCR_Full_BIT	= 42,
	ETHTOOL_LINK_MODE_10000baseSR_Full_BIT	= 43,
	ETHTOOL_LINK_MODE_10000baseLR_Full_BIT	= 44,
	ETHTOOL_LINK_MODE_10000baseLRM_Full_BIT	= 45,
	ETHTOOL_LINK_MODE_10000baseER_Full_BIT	= 46,
	ETHTOOL_LINK_MODE_2500baseT_Full_BIT	= 47,
	ETHTOOL_LINK_MODE_5000baseT_Full_BIT	= 48,

	ETHTOOL_LINK_MODE_FEC_NONE_BIT	= 49,
	ETHTOOL_LINK_MODE_FEC_RS_BIT	= 50,
	ETHTOOL_LINK_MODE_FEC_BASER_BIT	= 51,
	ETHTOOL_LINK_MODE_50000baseKR_Full_BIT		 = 52,
	ETHTOOL_LINK_MODE_50000baseSR_Full_BIT		 = 53,
	ETHTOOL_LINK_MODE_50000baseCR_Full_BIT		 = 54,
	ETHTOOL_LINK_MODE_50000baseLR_ER_FR_Full_BIT	 = 55,
	ETHTOOL_LINK_MODE_50000baseDR_Full_BIT		 = 56,
	ETHTOOL_LINK_MODE_100000baseKR2_Full_BIT	 = 57,
	ETHTOOL_LINK_MODE_100000baseSR2_Full_BIT	 = 58,
	ETHTOOL_LINK_MODE_100000baseCR2_Full_BIT	 = 59,
	ETHTOOL_LINK_MODE_100000baseLR2_ER2_FR2_Full_BIT = 60,
	ETHTOOL_LINK_MODE_100000baseDR2_Full_BIT	 = 61,
	ETHTOOL_LINK_MODE_200000baseKR4_Full_BIT	 = 62,
	ETHTOOL_LINK_MODE_200000baseSR4_Full_BIT	 = 63,
	ETHTOOL_LINK_MODE_200000baseLR4_ER4_FR4_Full_BIT = 64,
	ETHTOOL_LINK_MODE_200000baseDR4_Full_BIT	 = 65,
	ETHTOOL_LINK_MODE_200000baseCR4_Full_BIT	 = 66,
	ETHTOOL_LINK_MODE_100baseT1_Full_BIT		 = 67,
	ETHTOOL_LINK_MODE_1000baseT1_Full_BIT		 = 68,
	ETHTOOL_LINK_MODE_400000baseKR8_Full_BIT	 = 69,
	ETHTOOL_LINK_MODE_400000baseSR8_Full_BIT	 = 70,
	ETHTOOL_LINK_MODE_400000baseLR8_ER8_FR8_Full_BIT = 71,
	ETHTOOL_LINK_MODE_400000baseDR8_Full_BIT	 = 72,
	ETHTOOL_LINK_MODE_400000baseCR8_Full_BIT	 = 73,
	ETHTOOL_LINK_MODE_FEC_LLRS_BIT			 = 74,
	ETHTOOL_LINK_MODE_100000baseKR_Full_BIT		 = 75,
	ETHTOOL_LINK_MODE_100000baseSR_Full_BIT		 = 76,
	ETHTOOL_LINK_MODE_100000baseLR_ER_FR_Full_BIT	 = 77,
	ETHTOOL_LINK_MODE_100000baseCR_Full_BIT		 = 78,
	ETHTOOL_LINK_MODE_100000baseDR_Full_BIT		 = 79,
	ETHTOOL_LINK_MODE_200000baseKR2_Full_BIT	 = 80,
	ETHTOOL_LINK_MODE_200000baseSR2_Full_BIT	 = 81,
	ETHTOOL_LINK_MODE_200000baseLR2_ER2_FR2_Full_BIT = 82,
	ETHTOOL_LINK_MODE_200000baseDR2_Full_BIT	 = 83,
	ETHTOOL_LINK_MODE_200000baseCR2_Full_BIT	 = 84,
	ETHTOOL_LINK_MODE_400000baseKR4_Full_BIT	 = 85,
	ETHTOOL_LINK_MODE_400000baseSR4_Full_BIT	 = 86,
	ETHTOOL_LINK_MODE_400000baseLR4_ER4_FR4_Full_BIT = 87,
	ETHTOOL_LINK_MODE_400000baseDR4_Full_BIT	 = 88,
	ETHTOOL_LINK_MODE_400000baseCR4_Full_BIT	 = 89,
	ETHTOOL_LINK_MODE_100baseFX_Half_BIT		 = 90,
	ETHTOOL_LINK_MODE_100baseFX_Full_BIT		 = 91,
	/* must be last entry */
	__ETHTOOL_LINK_MODE_MASK_NBITS
};
#endif

#include <linux/phy.h>
static inline void _phy_copy_pause_bits(unsigned long *dst, unsigned long *src)
{
	linkmode_mod_bit_compat(ETHTOOL_LINK_MODE_Asym_Pause_BIT, dst,
		linkmode_test_bit_compat(ETHTOOL_LINK_MODE_Asym_Pause_BIT, *src));
	linkmode_mod_bit_compat(ETHTOOL_LINK_MODE_Pause_BIT, dst,
		linkmode_test_bit_compat(ETHTOOL_LINK_MODE_Pause_BIT, *src));
}

#ifndef RHEL8_BACKPORT_PHY_ADVERTISE_SUPPORTED
static inline void phy_advertise_supported(struct phy_device *phydev)
{
	#if KERNEL_VERSION(4, 20, 0) <= LINUX_VERSION_CODE
	DECLARE_BITMAP(new, __ETHTOOL_LINK_MODE_MASK_NBITS);
	#else
	unsigned long new = 0;
	#endif

	linkmode_copy(new, phydev->supported);
	phy_copy_pause_bits(new, phydev->advertising);
	linkmode_copy(phydev->advertising, new);
}
#endif // ! RHEL8_BACKPORT_PHY_ADVERTISE_SUPPORTED

#endif

#if  KERNEL_VERSION(4, 5, 0)  > LINUX_VERSION_CODE && !defined(RHEL7_BACKPORT_PHYDEV_NAME)
#include <linux/phy.h>

static inline const char *phydev_name(const struct phy_device *phydev)
{
	return dev_name(&phydev->dev);
}

#define ATTACHED_FMT "attached PHY driver [%s] (mii_bus:phy_addr=%s, irq=%d)"
static inline void phy_attached_print(struct phy_device *phydev, const char *fmt, ...)
{
	if (!fmt) {
		dev_info(&phydev->dev, ATTACHED_FMT "\n",
			 phydev->drv->name, phydev_name(phydev),
			 phydev->irq);
	} else {
		va_list ap;

		dev_info(&phydev->dev, ATTACHED_FMT,
			 phydev->drv->name, phydev_name(phydev),
			 phydev->irq);

		va_start(ap, fmt);
		vprintk(fmt, ap);
		va_end(ap);
	}
}
static inline void phy_attached_info(struct phy_device *phydev)
{
	phy_attached_print(phydev, NULL);
}
#endif // KERNEL_VERSION(4, 5, 0)  > LINUX_VERSION_CODE

#if  KERNEL_VERSION(4, 3, 0)  > LINUX_VERSION_CODE && !defined(RHEL7_BACKPORT_PHYDEV_REMOVE)
#include <linux/phy.h>
static inline void phy_device_remove(struct phy_device *phydev)
{
	struct mii_bus *bus = phydev->bus;
	int addr = phydev->addr;

	device_del(&phydev->dev);
	bus->phy_map[addr] = NULL;
}
#endif // KERNEL_VERSION(4, 3, 0)  > LINUX_VERSION_CODE

#if KERNEL_VERSION(3, 17, 0)  > LINUX_VERSION_CODE && !defined(RHEL7_BACKPORT_PHYDEV_SUSPEND)
#include <linux/phy.h>

static inline int phy_suspend_compat(struct phy_device *phydev)
{
	struct phy_driver *phydrv = to_phy_driver(phydev->dev.driver);
	struct ethtool_wolinfo wol;

	/* If the device has WOL enabled, we cannot suspend the PHY */
	wol.cmd = ETHTOOL_GWOL;
	phy_ethtool_get_wol(phydev, &wol);
	if (wol.wolopts)
		return -EBUSY;

	if (phydrv->suspend)
		return phydrv->suspend(phydev);
	return 0;
}

static inline int phy_resume_compat(struct phy_device *phydev)
{
	struct phy_driver *phydrv = to_phy_driver(phydev->dev.driver);

	if (phydrv->resume)
		return phydrv->resume(phydev);
	return 0;
}
#else

#define phy_suspend_compat phy_suspend
#define phy_resume_compat phy_resume

#endif // KERNEL_VERSION(3, 14, 0)  > LINUX_VERSION_CODE


//resources

#if KERNEL_VERSION(3, 1, 0) > LINUX_VERSION_CODE
/* helpers to define resources */
#define DEFINE_RES_NAMED(_start, _size, _name, _flags)			\
	{								\
		.start = (_start),					\
		.end = (_start) + (_size) - 1,				\
		.name = (_name),					\
		.flags = (_flags),					\
	}

#define DEFINE_RES_IO_NAMED(_start, _size, _name)			\
	DEFINE_RES_NAMED((_start), (_size), (_name), IORESOURCE_IO)
#define DEFINE_RES_IO(_start, _size)					\
	DEFINE_RES_IO_NAMED((_start), (_size), NULL)

#define DEFINE_RES_MEM_NAMED(_start, _size, _name)			\
	DEFINE_RES_NAMED((_start), (_size), (_name), IORESOURCE_MEM)
#define DEFINE_RES_MEM(_start, _size)					\
	DEFINE_RES_MEM_NAMED((_start), (_size), NULL)

#define DEFINE_RES_IRQ_NAMED(_irq, _name)				\
	DEFINE_RES_NAMED((_irq), 1, (_name), IORESOURCE_IRQ)
#define DEFINE_RES_IRQ(_irq)						\
	DEFINE_RES_IRQ_NAMED((_irq), NULL)


#endif

// dma-mapping.h
#if KERNEL_VERSION(3, 13, 0) > LINUX_VERSION_CODE && !defined(RHEL7_BACKPORT_COMPAT)
static inline int dma_set_mask_and_coherent(struct device *dev, u64 mask)
{
	int rc = dma_set_mask(dev, mask);
	if (rc == 0)
		dma_set_coherent_mask(dev, mask);
	return rc;
}
#endif


// of.h
#if KERNEL_VERSION(4, 2, 0) > LINUX_VERSION_CODE && !defined(RHEL7_BACKPORT_DEV_OF_NODE)
#include <linux/device.h>
#if defined(CONFIG_OF)
#include <linux/of.h>
static inline struct device_node * dev_of_node(struct device *dev){
	if (!IS_ENABLED(CONFIG_OF) || !dev)
		return NULL;
	return dev->of_node;
}
#else
static inline void * dev_of_node(struct device *dev) {
	(void)dev;
	return NULL;
}
#endif // defined(CONFIG_OF)
#endif // KERNEL_VERSION(4, 2, 0)

#include <linux/of.h>
#if !defined(of_match_ptr)
#define of_match_ptr(_ptr) NULL
#endif

#if KERNEL_VERSION(3, 9, 0) > LINUX_VERSION_CODE
#include <linux/device.h>
#include <linux/io.h>

static inline void __iomem * devm_ioremap_resource(struct device *dev, struct resource *res) {
	devm_request_mem_region(dev, res->start, resource_size(res), dev_name(dev));
	return devm_ioremap(dev, res->start, resource_size(res));
}
#endif // KERNEL_VERSION(3, 9, 0)


// pcie
#if  KERNEL_VERSION(4, 8, 0) > LINUX_VERSION_CODE && !defined(RHEL7_BACKPORT_PCI_IRQ_VECTOR)
#include <linux/pci.h>
static inline int pci_irq_vector(struct pci_dev *dev, unsigned int nr)
{
	if (WARN_ON_ONCE(nr > 0))
		return -EINVAL;
	return dev->irq;
}
static inline void pci_free_irq_vectors(struct pci_dev *dev)
{
}
#endif

//interrupts

// compatiblity for ~2.6.36
#ifndef IRQF_DISABLED
/// defines to select kernel version dependend code parts
#define IRQF_DISABLED 0x0
#endif

#if  KERNEL_VERSION(5, 5, 0) > LINUX_VERSION_CODE && !defined(SUSE15SP2_BACKPORT_DEVM_IOREMAP)
#include <linux/platform_device.h>
//platform_driver
static inline void __iomem *
devm_platform_ioremap_resource_byname(struct platform_device *pdev,
					  const char *name)
{
	struct resource *res;

	res = platform_get_resource_byname(pdev, IORESOURCE_MEM, name);
	if (!res)
		return ERR_PTR(-EINVAL);
	return devm_ioremap_resource(&pdev->dev, res);
}
#endif


// spi driver remove return type
#if  KERNEL_VERSION(5, 18, 0) <= LINUX_VERSION_CODE
#define SPI_DRIVER_REMOVE_RETURN_TYPE void
#define SPI_DRIVER_REMOVE_RETURN_VALUE
#else
#define SPI_DRIVER_REMOVE_RETURN_TYPE int
#define SPI_DRIVER_REMOVE_RETURN_VALUE 0
#endif

// i2c driver remove return type
#if  KERNEL_VERSION(6, 1, 0) <= LINUX_VERSION_CODE
#define I2C_DRIVER_REMOVE_RETURN_TYPE void
#define I2C_DRIVER_REMOVE_RETURN_VALUE
#else
#define I2C_DRIVER_REMOVE_RETURN_TYPE int
#define I2C_DRIVER_REMOVE_RETURN_VALUE 0
#endif
#if  KERNEL_VERSION(6, 3, 0) <= LINUX_VERSION_CODE
#define I2C_DRIVER_PROBE_OTHER_PARAMS
#else
#define I2C_DRIVER_PROBE_OTHER_PARAMS ,const struct i2c_device_id *device_id
#endif



// pci.h

#include <linux/pci.h>
#ifndef PCI_DEVICE_DATA
#define PCI_DEVICE_DATA(vend, dev, data) \
	.vendor = PCI_VENDOR_ID_##vend, .device = PCI_DEVICE_ID_##vend##_##dev, \
	.subvendor = PCI_ANY_ID, .subdevice = PCI_ANY_ID, 0, 0, \
	.driver_data = (kernel_ulong_t)(data)
#endif // PCI_DEVICE_DATA

#endif // _SYN1588_COMPATIBILITY_H
