/* SPDX-License-Identifier: GPL-2.0
 *
 * Copyright (C) 2020 Norbert Summer <summer@oregano.at>. All Rights Reserved.
 */

// this file garanties compatibility between different kernel versions for RHEL backports

#ifndef _SYN1588_COMPATIBILITY_RHEL_H
#define _SYN1588_COMPATIBILITY_RHEL_H

#include <linux/version.h>

#ifdef RHEL_RELEASE_CODE

#ifndef RHEL_RELEASE_VERSION
#define RHEL_RELEASE_VERSION(a,b) (((a) << 8) + (b))
#endif

#if  (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(6,4)) && \
	 (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(7,0))
#define RHEL6_ETHTOOL_OPS_EXT_STRUCT
#define RHEL_BACKPORT_IDR2
#endif /* RHEL >= 6.4 && RHEL < 7.0 */

#if  (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(7,0)) && \
	 (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(8,0))
#define RHEL7_BACKPORT_COMPAT
#endif /* RHEL >= 7.0 && RHEL < 8.0 */

#if  (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(7,2)) && \
	 (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(8,0))
#define RHEL7_BACKPORT_DEV_OF_NODE
#define RHEL7_BACKPORT_PTP_SETTIME64
#endif /* RHEL >= 7.2 && RHEL < 8.0 */

#if  (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(7,4)) && \
	 (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(8,0))
#define RHEL7_BACKPORT_PHYDEV_REMOVE
#define RHEL7_BACKPORT_PHYDEV_SUSPEND
#endif /* RHEL >= 7.4 && RHEL < 8.0 */

#if  (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(7,5)) && \
	 (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(8,0))
#define RHEL7_BACKPORT_PCI_IRQ_VECTOR
#endif /* RHEL >= 7.5 && RHEL < 8.0 */

#if  (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(7,7)) && \
	 (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(8,0))
#define RHEL7_BACKPORT_PHYDEV_NAME
#endif /* RHEL >= 7.7 && RHEL < 8.0 */

#if  (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(8,0)) && \
	 (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(9,0))
#define RHEL8_BACKPORT_NDO_TX_TIMEOUT
#endif /* RHEL >= 8.0 && RHEL < 9.0 */

#if  (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(8,2)) && \
     (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(9,0))
#define RHEL8_BACKPORT_PHY_ADVERTISE_SUPPORTED
#endif /* RHEL >= 8.2 && RHEL < 9.0 */

#if  (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(8,3)) && \
	 (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(9,0))
#define RHEL8_BACKPORT_PHY_IOCTL
#endif /* RHEL >= 8.3 && RHEL < 9.0 */

#if  (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(8,4)) && \
	 (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(9,0))
#define RHEL8_BACKPORT_LINKMODE_TEST_BIT
#endif /* RHEL >= 8.4 && RHEL < 9.0 */

#if  (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(8,7)) && \
	 (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(9,0))
#define RHEL_BACKPORT_ETHTOOL_RINGPARAM
#endif /* RHEL >= 8.7 && RHEL < 9.0 */

#if  (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(9,0)) && \
	 (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(10,0))
#define RHEL9_BACKPORT_NDO_ETH_IOCTL
#endif /* RHEL >= 9.0 && RHEL < 10.0 */

#if  (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(9,1)) && \
	 (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(10,0))
#define RHEL_BACKPORT_ETHTOOL_RINGPARAM
#endif /* RHEL >= 91 && RHEL < 10.0 */

#endif // defined(RHEL_RELEASE_CODE)
#endif //_SYN1588_COMPATIBILITY_RHEL_H
