/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Oregano SPI header file
 * based on altera spi
 */
#ifndef OREGANO_SPI_H
#define OREGANO_SPI_H

#include <linux/interrupt.h>
#include <linux/spi/spi.h>
#include <linux/types.h>

#define OREGANO_SPI_MAX_CS		32

/**
 * struct oregano_spi_platform_data - Platform data of the Oregano SPI driver
 * @mode_bits:		Mode bits of SPI master.
 * @num_chipselect:	Number of chipselects.
 * @num_devices:	Number of devices that shall be added when the driver
 *			is probed.
 * @clock_hz		speed of the SPI bus
 * @devices:		The devices to add.
 */
struct oregano_spi_platform_data {
	u16				mode_bits;
	u16				num_chipselect;
	u32				clock_hz;
	u16				num_devices;
	struct spi_board_info		*devices;
};

#endif /* OREGANO_SPI_H */
