
#ifndef _CLOCK_M_H_
#define _CLOCK_M_H_

#include "compat/time.h"
#include "include/syn1588_sys.h"
#include "si_time_pll.h"

#include <linux/io.h>
#include <linux/interrupt.h>
#include <linux/irqreturn.h>
#include <linux/ptp_clock_kernel.h>
#include <linux/spinlock_types.h>
#include <linux/types.h>


struct syncD_device;
struct clock_m_device;

struct oregano_clock_platform_data {
	struct si_time_95147 *si_time_pll;
};

struct oregano_clock_event_data {
	u32 irs; // interrupt source register
};

struct oregano_clock_callback {
	void(*fn)(const struct clock_m_device *, const struct oregano_clock_event_data *, void *data);
	void *data;
};

/// CLOCK_M (clock synchronization cell) private data structure (CLOCK_M device)
struct clock_m_device {
	void __iomem *iobase; ///< pointer to mapped PCI I/O memory
	struct resource *resource;
	int irq; // hardware irq id
	struct device *dev;

	// ptp structs
	struct ptp_clock *ptp_clock;
	struct ptp_clock_info ptp_clock_info;

	struct si_time_95147 *si_time_pll;

	// pointer to clock_m for extime funktions
	struct clock_m_device *extime_clock_m;

	char version[5]; ///< CLOCK_M version number
	bool ntpModeEnabled; ///< non zero if NTP mode is enabled

	struct oregano_clock_callback *callback; // TODO handle as list

	struct syn1588_io *syn1588_io;


	/**
	 * see  `CLOCK_M_CAPABILITY_*` in `an001_register_map`
	 * for more information
	 */
	struct oregano_clock_m_capabilities {
		bool miits0;
		bool miits1;
		bool lsync_sync_status;
		bool measurement;
		bool event0;
		bool event1;
		bool trigger0;
		bool trigger1;
		bool period0;
		bool period1;
		bool time2;
		bool irigb_output;
		bool onestep;
		bool period0cfg;
		bool ntp;
		bool leapset;
		bool dutyperiod;
		bool packetgenerator;
		bool packetslaveformat;
		bool precisionperiod;
		bool irigb_input;
		bool ext_vlan_pgkgen;
		bool stop_clock;
		bool hsnetworkindirect;
		bool period2;
		bool period3;
		bool extime;
		bool ptp_sync_status;
		bool sdi_extended_support;
		bool audio_counter;
		bool sdi_ltc_timecode;

		bool miits2_1g;
		bool miits3_1g;
		bool miits4_1g;
		bool miits5_1g;
		bool miits2_10g;
		bool miits3_10g;
		bool miits4_10g;
		bool miits5_10g;
		bool miits2_25g;
		bool miits3_25g;
		bool miits4_25g;
		bool miits5_25g;
		bool miits2_40g;
		bool miits3_40g;
		bool miits4_40g;
		bool miits5_40g;
		bool miits2_100g;
		bool miits3_100g;
		bool miits4_100g;
		bool miits5_100g;
	} clock_m_capabilities;

	// depricated api
	struct syncD_device *syncd_dev; ///< device info pointer for callback

	unsigned long lost_irqs; ///< lost IRQ mask (IRQ handler -> BH handler)

	spinlock_t reg_lock_eventctrl;
	spinlock_t reg_lock_iomatrix;

	// virtual registers
	spinlock_t reg_lock; ///< used to lock virtual registers and write verify


	spinlock_t time1_lock; // lock register time1
	spinlock_t time2_lock; // lock register time2

	struct tasklet_struct tl_clock_m; ///< bottom-half handler for this device

	unsigned long
		irh_irsrc; ///< virtual IRSRC register for IRQ handler
	unsigned long
		bhh_irsrc; ///< virtual IRSRC register for bottom half handler
	u32 chr_irsrc; ///< virtual IRSRC register for character device
	u32 clk_freq; ///< virtual CORE_FREQ register (contains core clock
		/// frequency [Hz])
};


struct clock_m_device *dev_get_clock_m(struct device *);

void set_mcore_phy_delay(struct clock_m_device *adapter, u32 tx_delay, u32 rx_delay);

int clock_m_io_probe(struct clock_m_device *clock_m);
int clock_m_period_setup (
	struct clock_m_device *clock_m,
	enum syn1588_ioctl_period_id period_id,
	struct syn1588_ioctl_duration high_time,
	struct syn1588_ioctl_duration low_time,
	bool is_rising_edge,
	enum syn1588_ioctl_period_start_condition start_condition
);
int clock_m_period_clear(
	struct clock_m_device *clock_m,
	enum syn1588_ioctl_period_id period_id
);
int clock_m_period_set_enable(
	struct clock_m_device *clock_m,
	enum syn1588_ioctl_period_id period_id,
	bool value
);
int clock_m_period_get_durations(
	struct clock_m_device *clock_m,
	enum syn1588_ioctl_period_id period_id,
	struct syn1588_ioctl_duration *high_time,
	struct syn1588_ioctl_duration *low_time
);

int clock_m_event_setup (
	struct clock_m_device *clock_m,
	enum syn1588_ioctl_event_id event_id,
	wait_queue_head_t *wait_queue
);

int clock_m_event_free(
	struct clock_m_device *clock_m,
	enum syn1588_ioctl_event_id event_id
);

int clock_m_event_clear(
	struct clock_m_device *clock_m,
	enum syn1588_ioctl_event_id event_id
);

int clock_m_event_ts_available(
	struct clock_m_device *clock_m,
	enum syn1588_ioctl_event_id event_id
);

/**
 * returnes next aviable timestamp if there is one otherwise -EAGIAN
*/
int clock_m_event_get_ts(
	struct clock_m_device *clock_m,
	enum syn1588_ioctl_event_id event_id,
	syn1588_ioctl_timepoint *buffer
);

/**
 * read timestamps from the hardware and put it into the ringbuffer
 * this function should be polled or used normaly by the irq of the clock_m
 */
int clock_m_event_read_ts(
	struct clock_m_device *clock_m,
	enum syn1588_ioctl_event_id event_id
);

int clock_m_trigger_setup (
	struct clock_m_device *clock_m,
	enum syn1588_ioctl_trigger_id trigger_id
);

int clock_m_trigger_clear(
	struct clock_m_device *clock_m,
	enum syn1588_ioctl_trigger_id trigger_id
);

/**
 * add new trigger points to the ring buffer
 * the list must be sorted ascending
*/
int clock_m_trigger_add_action(
	struct clock_m_device *clock_m,
	enum syn1588_ioctl_trigger_id trigger_id,
	struct syn1588_ioctl_trigger_entry *buffer
);

/**
 * writes data to the hardware as much as currently possible
 * this function should be called by the irq function
 */
int clock_m_trigger_write_ts(
	struct clock_m_device *clock_m,
	enum syn1588_ioctl_trigger_id trigger_id
);

int clock_m_io_port_setup (
	struct clock_m_device *clock_m,
	enum syn1588_ioctl_io_port_id io_port_id
);

int clock_m_io_port_clear(
	struct clock_m_device *clock_m,
	enum syn1588_ioctl_io_port_id io_port_id
);
int clock_m_io_matrix_connect(
	struct clock_m_device *clock_m,
	struct syn1588_ioctl_source source_ioctl,
	struct syn1588_ioctl_sink sink_ioctl
);
int clock_m_io_matrix_disconnect(
	struct clock_m_device *clock_m,
	struct syn1588_ioctl_source source_ioctl,
	struct syn1588_ioctl_sink sink_ioctl
);
int clock_m_io_matrix_remove_sink(
	struct clock_m_device *clock_m,
	struct syn1588_ioctl_sink sink_ioctl
);
int clock_m_io_matrix_remove_source(
	struct clock_m_device *clock_m,
	struct syn1588_ioctl_source source_ioctl
);

// exported functions for Synchronization Driver
int clock_m_getreg(struct clock_m_device *, unsigned long address, u32 *data);
int clock_m_setreg(struct clock_m_device *, unsigned long address, u32 data);
bool clock_m_setreg_verify(struct clock_m_device *clock_m, unsigned long address, u32 data);
int clock_m_getTimestamps(struct clock_m_device *clock_m_dev, syn1588_timespec *sys_ts,
			  syn1588_timespec *hw_ts, unsigned long long *delta_sys,
			  unsigned long long *delta_hw);


//io matrix




/**
 * register a new callback for clock_m events
 * @param callback caller need to handle memory until `clock_m_unregister_callback`
 * @return on success 0, otherwise -<Errno>
 */
int clock_m_register_callback(struct clock_m_device *, struct oregano_clock_callback*callback);

/**
 * unregister a existing callback for clock_m events
 * @param callback same pointer as used in `clock_m_register_callback`
 * @return on success 0, otherwise -<Errno>
 */
int clock_m_unregister_callback(struct clock_m_device *, struct oregano_clock_callback*callback);



#endif // _CLOCK_M_H_
