/* SPDX-License-Identifier: GPL-2.0
 *
 * Copyright (C) 2020 Norbert Summer <summer@oregano.at>. All Rights Reserved.
 */

#ifndef _SYN1588_COMPATIBILITY_TIME_H
#define _SYN1588_COMPATIBILITY_TIME_H

#include <linux/timer.h>
#include <linux/version.h>

#if (KERNEL_VERSION(3, 17, 0) <= LINUX_VERSION_CODE)
// support of 64bit kernel time

#include <linux/timekeeping.h>

// alias for current used timestamp
typedef struct timespec64 syn1588_timespec;
typedef time64_t syn1588_time;

#define SYN1588_TIMESPEC_PRINTF_FORMAT "%llu"

static inline syn1588_timespec syn1588_timespec_sub(syn1588_timespec lhs,
							syn1588_timespec rhs)
{
	return timespec64_sub(lhs, rhs);
}

/**
 * @brief small wrapper for getting current unix time from kernel
 *
 * @param ts writes time into this variable
 */
static inline void syn1588_ktime_get_real_ts(syn1588_timespec *ts)
{
	ktime_get_real_ts64(ts);
}

#else
// no 64bit timespec support

typedef struct timespec syn1588_timespec;
typedef time_t syn1588_time;

#define SYN1588_TIMESPEC_PRINTF_FORMAT "%lu"

static inline syn1588_timespec syn1588_timespec_sub(syn1588_timespec lhs,
							syn1588_timespec rhs)
{
	return timespec_sub(lhs, rhs);
}

static inline void syn1588_ktime_get_real_ts(syn1588_timespec *ts)
{
	ktime_get_real_ts(ts);
}

#endif



#endif // _SYN1588_COMPATIBILITY_TIME_H
