/* SPDX-License-Identifier: GPL-2.0
--------------------------------------------------------------------------------
--
--	syn1588(R) NIC
--
--------------------------------------------------------------------------------
--
--	Oregano Systems - Design & Consulting GesmbH
--
--	Web: http://oregano.at
--	@: support@oregano.at
--
--------------------------------------------------------------------------------
--
--	Copyright (c) 2006-2021
--
--------------------------------------------------------------------------------
--
--	Confidential data.
--
-- This source code, netlists, documentation is confidential and must not be
-- published nor made available to third parties. Usage of this source code,
-- netlists, documentation is subject to restrictions and conditions of a
-- license agreement.
--
-- ALL RIGHTS RESERVED.
-- Oregano Systems does not assume any liability
-- arising out of the application or use of any product described or shown
-- herein nor does it convey any license under its patents, copyrights, or any
-- rights of others.
--
-- Oregano Systems reserves the right to make changes, at any time, in
-- order to improve reliability, function or design. Oregano Systems will not
-- assume responsibility for the use of any circuitry described herein.
--
-- All trademarks used are the property of their respective owners.
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-- MA	02110-1301, USA.
--
--------------------------------------------------------------------------------
--
--	Author:	Patrick Loschmidt
--	 Austrian Academy of Science
--	 Institute for Integrated Sensor Systems
--	Date of Creation:			 Fri 2006-06-16
--	Description: register definitions header file for
--	 Syn1588 PCI-NIC device driver
-------------------------------------------------------------------------------
*/

#ifndef _CHAR_SYN1588NIC_REG_
#define _CHAR_SYN1588NIC_REG_

enum pci_offset {
	PCI_OFFSET_MAC = 0x0000,
	//PCI_OFFSET_MAC_I2C = 0x0000,
	PCI_OFFSET_MAC_MAIN = 0x0000,
	PCI_OFFSET_CLOCK_M = 0x1000,
	PCI_OFFSET_SPI_PLL = 0x3000,
	PCI_OFFSET_FPGA_UPDATE = 0x3040,
	PCI_OFFSET_SPI_DAC_OXCO = 0x30C0,
	PCI_OFFSET_I2C_VIDEO_DAC = 0x30C0,
	PCI_OFFSET_EEPROM = 0x3100,
	PCI_OFFSET_SYSTEM_REGISTER_BLOCK = 0x5200,
	PCI_OFFSET_PCI_BRIDGE = 0x8000,
};

#define PCI_SIZE_SYN1588_NIC 0x10000
#define PCI_SIZE_MAC_I2C 0x10
#define PCI_SIZE_MAC 0x1000
#define PCI_SIZE_CLOCK_M 0x1000
#define PCI_SIZE_SPI_PLL 0x40
#define PCI_SIZE_FPGA_UPDATE 0x40
#define PCI_SIZE_I2C_VIDEO_DAC 0x40
#define PCI_SIZE_EEPROM 0x40
#define PCI_SIZE_SYSTEM_REGISTER_BLOCK 0x100
#define PCI_SIZE_PCI_BRIDGE 0x2000

// eeprom board capabilities for rev 2.3+
enum syn1588_eeprom {
	SYN1588_EEPROM_SERIAL_NR = 0x10,
	SYN1588_EEPROM_BOARD_CAPABILITIES = 0x20,
};

enum syn1588_eeprom_board_capabilities {
	SYN1588_EEPROM_BOARD_CAPABILITIES_TIMECODE = BIT(24),
	SYN1588_EEPROM_BOARD_CAPABILITIES_OCXO = BIT(25),
	SYN1588_EEPROM_BOARD_CAPABILITIES_FPGA_TYPE = BIT(26),
	SYN1588_EEPROM_BOARD_CAPABILITIES_VIDEO_DAC = BIT(27),
};

enum syn1588_system_register_block {
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY0 = 0x00,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY1 = 0x04,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY2 = 0x08,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY3 = 0x0c,
	SYN1588_SYSTEM_REGISTER_BLOCK_CONFIG0 = 0x10,
	SYN1588_SYSTEM_REGISTER_BLOCK_CONFIG1 = 0x14,
	SYN1588_SYSTEM_REGISTER_BLOCK_CONFIG2 = 0x18,
	SYN1588_SYSTEM_REGISTER_BLOCK_CONFIG3 = 0x1c,
	SYN1588_SYSTEM_REGISTER_BLOCK_BUILD_ID = 0x30,
};


enum syn1588_system_register_block_capability0 {
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY0_TSE_MAC = 0,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY0_CLOCKM = 1,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY0_PLL = 2,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY0_VIDEO_DAC = 3,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY0_GPS = 4,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY0_OXCO = 5,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY0_SFP = 6,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY0_SPI = 7,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY0_EXT_CLK_INPIT = 8,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY0_REV21 = 9,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY0_I2C_OSCILLATOR = 10,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY0_DUAL_NIC = 11,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY0_EXTIME = 12,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY0_MAC_SERIAL = 13,
	SYN1588_SYSTEM_REGISTER_BLOCK_CAPABILITY0_HIGH_SPEED_NIC = 14,
};


/******************************************************************************
 * syn1588(R)Clock_M Registers
 *****************************************************************************/
enum clock_m_register {
	SYN1588_VER = 0x000, ///< Syn1588(R) CLOCK_M version register R
	TSFIFO0RX_L = 0x010, ///< receive Timestamp FIFO0 data <31:00> R
	TSFIFO0TX_L = 0x018, ///< send Timestamp FIFO0 data <31:00> R
	TSFIFO1RX_L = 0x01C, ///< receive Timestamp FIFO1 data <31:00> R
	TSFIFO1TX_L = 0x020, ///< send Timestamp FIFO1 data <31:00> R
	TSFIFO0_STATUS =
		0x030, ///< Timestamp FIFO0 counter and full/empty flags R
	TSFIFO1_STATUS =
		0x034, ///< Timestamp FIFO1 counter and full/empty flags R
	IRSRC = 0x040, ///< Interrupt Source R/W
	IREN = 0x044, ///< Enables various interrupts R/W
	TIMECTRL = 0x048, ///< Controls operation of the local clock R/W
	EVENTCTRL = 0x04C, ///< Control of events R/W
	SHDWSTEPPU_L =
		0x050, ///< Future step size (pure phase), lower 32 bit R/W
	SHDWSTEPPU_H =
		0x054, ///< Future step size (pure phase), upper 32 bit R/W
	ONESTEPCTRL = 0x058, ///< One step control register
	PHY_DELAY_TX = 0x064, ///< PHY delay correction transmit, R/W
	PHY_DELAY_RX = 0x068, ///< PHY delay correction receive, R/W
	CAPABILITIES1 = 0x06C, ///< Indicates implemented hardware features, R
	CAPABILITIES2 = 0x070, ///< Reserved for future use, R
	//SHDWTIME_L = 0x080, ///< Future contents of the clock, first 32 bit W
	SHDWTIME_NS = 0x084, ///< Future contents of the clock, middle 32 bit W
	SHDWTIME_SEC = 0x088, ///< Future contents of the clock, upper 32 bit W
	//SHDWTIME_E = 0x08C, ///< Future contents of the clock, high 32 bit W
	TIME_L = 0x090, ///< Current time, lower 32 bit R (deprecated)
	TIME_NS = 0x094, ///< Current time, middle 32 bit R (equivalent to TIME_NS in syn1588ifc)
	TIME_SEC = 0x098, ///< Current time, upper 32 bit R (equivalent to TIME_SEC in syn1588ifc)
	EVENTTIME0_L = 0x0A4, ///< Time of last event 0, first 32 bit R
	EVENTTIME0_H = 0x0A8, ///< Time of last event 0, upper 32 bit R
	EVENTTIME1_L = 0x0B0, ///< Time of last event 1, first 32 bit R
	EVENTTIME1_H = 0x0B4, ///< Time of last event 1, upper 32 bit R
	FRAC_NUM = 0x0B8, ///< The FRAC_NUM register holds the value of "n+m" of the precision period function.
	FRAC_DENUM = 0x0BC, ///< This is the fractional counter denominator register of period 0.
	GENLOCK_CTRL = 0x0C8, ///< The genlock control register controls the operation of the precision period functions, pre-scaler of even inputs and video applications.
	TRIGTIME0_L = 0x0D0, ///< Time to trigger event 0, first 32 bit R/W
	TRIGTIME0_H = 0x0D4, ///< Time to trigger event 0, upper 32 bit R/W
	TRIGTIME1_L = 0x0D8, ///< Time to trigger event 1, first 32 bit R/W
	TRIGTIME1_H = 0x0DC, ///< Time to trigger event 1, upper 32 bit R/W
	PERIODTIME0_L = 0x0F0, ///< Period of timer, lower 32 bit R/W
	PERIODTIME0_H = 0x0F4, ///< Period of timer, upper 32 bit R/W
	PERIODTIME1_L = 0x100, ///< Period of timer, lower 32 bit R/W
	PERIODTIME1_H = 0x104, ///< Period of timer, upper 32 bit R/W
	LEAP_SEC =
		0x108, ///< Current leap seconds (8 bit) R (current value / W (future value)
	LEAP_APPLYTIME =
		0x10C, ///< Leap seconds apply time (32 bit seconds) R/W
	OSZPWM_CTR = 0x1E0, ///< PWM output control register R/W
	CLOCKID_L = 0x1E4, ///< Low word of the 64 bit EUI R
	CLOCKID_H = 0x1E8, ///< High word of the 64 bit EUI R
	IOMATRIX = 0x200, ///< I/O interconnection definition R/W
	MIITS0_TXCTRL = 0x208, ///< MII Timestamper 0 TX control
	MIITS1_RXCTRL = 0x20C, ///< MII Timestamper 1 RX control
	MIITS1_TXCTRL = 0x210, ///< MII Timestamper 1 TX control
	MIITS0_RXCTRL = 0x214, ///< MII Timestamper 0 RX control
	TIME2_M = 0x220, ///< Current time, middle 32 bit R
	TIME2_H = 0x224, ///< Current time, middle 32 bit R
	MIITS0_RXMEM = 0x400, ///< MII Timestamper 0 RX pattern memory
	MIITS0_TXMEM = 0x600, ///< MII Timestamper 0 TX pattern memory
	MIITS1_RXMEM = 0x800, ///< MII Timestamper 1 RX pattern memory
	MIITS1_TXMEM = 0xA00, ///< MII Timestamper 1 TX pattern memory
	/*
	 * Virtual registers
	 */
	CLK_FREQ =
		0x204, ///< Syn1588(R) CLOCK_M frequency [Hz]
	PERIODTIME2_L = 0x254, ///< Period of timer, lower 32 bit R/W
	PERIODTIME2_H = 0x258, ///< Period of timer, upper 32 bit R/W
	PERIODTIME3_L = 0x25C, ///< Period of timer, lower 32 bit R/W
	PERIODTIME3_H = 0x260, ///< Period of timer, upper 32 bit R/W
	HWBUILD =
		0x1F0, ///< Virtual copy of MAC_ETH_BUILD, DEPRECATED, USE VR REGISTERS
	VR_CLK_FREQ = 0x1000, ///< Syn1588(R) CLOCK_M frequency [Hz]
	VR_ETH_VERSION = 0x1004, ///< Virtual copy of MAC_ETH_VERSION
	VR_ETH_BUILD = 0x1008, ///< Virtual copy of MAC_ETH_BUILD
	VR_PCI_REVISION = 0x100C, ///< Virtual copy of PCI revision
	// MAC TS registers
	VR_ETH_TSFIFORX = 0x1010, ///< MAC receive timestamp FIFO data
	VR_ETH_TSFIFOTX = 0x1014, ///< MAC transmit timestamp FIFO data
	VR_ETH_TSFIFO_STATUS =
		0x1018, ///< Four 6 bit counter in the MAC TSFIFO and full/empty flags
	VR_ETH_TS_TXCTRL =
		0x1020, ///< MAC transmit timestamper control register
	VR_ETH_TS_RXCTRL = 0x1024, ///< MAC receive timestamper control register
	VR_ETH_PHY_DELAY_TX =
		0x1028, ///< Correction values for compensating PHY transmit delay
	VR_ETH_PHY_DELAY_RX =
		0x102C, ///< Correction values for compensating PHY receive delay
	VR_ETH_TIMECTRL = 0x1030, ///< MAC time control register
	VR_ETH_1STEPCTRL = 0x1034, ///< MAC one step control
	VR_ETH_TS2_TXCTRL = 0x1040, ///< 2nd MAC Timestamper RX Control
	VR_ETH_TS2_RXCTRL = 0x1044, ///< 2nd MAC Timestamper TX Control
	VR_ETH_TS2_1STEPCTRL =
		0x1054, ///< Allows to set different modes for 2nd MAC one step timestamper
	VR_ETH_TS_RXMEM_BASE =
		0x1100, ///< Base address of MAC receive timestamper mask/pattern memory
	VR_ETH_TS2_RXMEM_BASE =
		0x1200, ///< Base address of 2nd MAC Timestamper RX Pattern/Mask Memory
	VR_ETH_TS_TXMEM_BASE =
		0x1300, ///< Base address of MAC transit timestamper mask/pattern memory
	VR_ETH_TS2_TXMEM_BASE =
		0x1400, ///< Base address of 2nd MAC Timestamper TX Pattern/Mask Memory
	// Error Counter
	VR_ETH_RXERR = 0x1500, ///< Virtual copy of MAC_ETH_CNT_RXERR
	VR_ETH_FCSERR = 0x1504, ///< Virtual copy of MAC_ETH_CNT_FCSERR
	VR_ETH_MCASTERR = 0x1508, ///< Virtual copy of MAC_ETH_CNT_MCASTERR
	VR_ETH_LCOLLERR = 0x150C, ///< Virtual copy of MAC_ETH_CNT_LCOLLERR
	VR_ETH_LENERR = 0x1510, ///< Virtual copy of MAC_ETH_CNT_LENERR
	VR_ETH_OVERERR = 0x1514, ///< Virtual copy of MAC_ETH_CNT_OVERERR
	VR_ETH_FRAMEERR = 0x1518, ///< Virtual copy of MAC_ETH_CNT_FRAMEERR
	VR_ETH_RX = 0x151C, ///< Virtual copy of MAC_ETH_CNT_RX
	VR_ETH_TX = 0x1520, ///< Virtual copy of MAC_ETH_CNT_TX
	VR_ETH_CNT_VLANERR = 0x1524, ///< Virtual copy of MAC_CNT_VLAN_ERR
	VR_ETH_CNT_VLAN = 0x1528, ///< Virtual copy of MAC_CNT_VLAN
	VR_ETH_CNT_BROADCAST = 0x152C, ///< Virtual copy of MAC_CNT_BROADCAST
	VR_ETH_CNT_MULTICAST = 0x1530, ///< Virtual copy of MAC_CNT_MULTICAST
	VR_ETH_CNT_RX_TS = 0x1534, ///< Virtual copy of MAC_CNT_RX_TS
	VR_ETH_CNT_TX_TS = 0x1538, ///< Virtual copy of MAC_CNT_TX_TS
	VR_ETH_CNT_RX_TS2 = 0x153C, ///< Virtual copy of MAC_CNT_RX_TS2
	VR_ETH_CNT_TX_TS2 = 0x1540, ///< Virtual copy of MAC_CNT_TX_TS2
	// MAC Capabilities
	VR_ETH_CAPABILITIES_0 = 0x1544, ///< Virtual copy of MAC_CAPABILITY_0
	VR_ETH_MIICOMMAND = 0x1548, ///< Virtual copy of MAC_MIICOMMAND
	VR_ETH_SFP_EEPROM = 0x154C, ///< Virtual copy of MAC_SFP_EEPROM
	VR_ETH_CAPABILITIES_1 = 0x1550, ///< Virtual copy of MAC_CAPABILITY_1
	VR_ETH_CAPABILITIES_2 = 0x1554, ///< Virtual copy of MAC_CAPABILITY_2
	VR_ETH_CAPABILITIES_3 = 0x1558, ///< Virtual copy of MAC_CAPABILITY_3
	VR_ETH_IFG_CTRL = 0x155C, ///< Virtual copy of MAC_IFG_CTRL

	// lSync
	VR_SYSTS_L = 0x1600, ///< Nanoseconds of system timestamp
	VR_SYSTS_H = 0x1604, ///< Seconds of system timestamp
	VR_HWTS_L = 0x1608, ///< Nanoseconds of hardware timestamp
	VR_HWTS_H = 0x160C, ///< Seconds of hardware timestamp
	VR_DELTASYS = 0x1610, ///< Clock cycles to draw system timestamp
	VR_DELTAHW = 0x1614, ///< Clock cycles to draw hardware timestamp

	// customer specific
	VR_AST_AUXR = 0x1800, ///< MBG specific
	VR_TSU_SLOT_ID = 0x1804, ///< MBG specific
	VR_TSU_PHY_INFO = 0x1808, ///< MBG specific
	VR_AST_RESET_CNTL = 0x180C, ///< MBG specific
	VR_AST_MUX_CNTL = 0x1810, ///< MBG specific
	VR_SOC_FPGA_VERSION = 0x1814, ///< MBG specific

	/*
	 * SPI Controllers for FLASH, PLL and DAC
	 */
	SPI_PLL_CTRL = 0x2000,
	SPI_PLL_DATA = 0x2004,
	SPI_CONFIG_CTRL = 0x2040,
	SPI_CONFIG_DATA = 0x2044,
	SPI_DAC_CTRL = 0x2080,
	SPI_DAC_DATA = 0x2084,

	/*
	 * PktGen Registers
	 */
	PKT_GEN_CTRL = 0x3000,
	PKT_GEN_SL_RAM_LINE_L = 0x3004,
	PKT_GEN_SL_RAM_LINE_H = 0x3008,
	PKT_GEN_DATA = 0x300C,
	PKT_GEN_SYNC_SEQID_L = 0x3010,
	PKT_GEN_SYNC_SEQID_H = 0x3014,
	PKT_GEN_ANN_SEQID_L = 0x3018,
	PKT_GEN_ANN_SEQID_H = 0x301C,
	PKT_GEN_PKT_MODE = 0x3020,
	PKT_GEN_MAX_SYNC_RATE = 0x3024,
	PKT_GEN_SLAVE_LIST_L = 0x3028,
	PKT_GEN_SLAVE_LIST_H = 0x302C,
	PKT_GEN_ANN_LEN = 0x3030,
	PKT_GEN_STATUS_L = 0x3034,
	PKT_GEN_STATUS_H = 0x3038,

};

/// wrap appropriate registers for driver default test register names
enum test_reg_map {
	TESTPERIOD0_H = PERIODTIME0_H,
	TESTPERIOD1_H = PERIODTIME1_H,
	TESTTRIG0_H = TRIGTIME0_H,
	TESTTRIG1_H = TRIGTIME1_H
};
/// valid address range for synchronization driver
// TODO: integrate these into enum clock_m_register above, or derive those
//			 values from enum clock_m_register
#define CLOCK_M_ADDR_RANGE 0xC04

#define CLOCK_M_VR_ADDR_RANGE_L 0x1000
#define CLOCK_M_VR_ADDR_RANGE_H 0x1FFF
#define CLOCK_M_SPI_ADDR_RANGE_L 0x2000
#define CLOCK_M_SPI_ADDR_RANGE_H 0x2100
#define CLOCK_M_MAC_SER_NUM_ADDR_RANGE_L 0x2100
#define CLOCK_M_MAC_SER_NUM_ADDR_RANGE_H 0x213F

/// offset and range for the pktgen v2.0
#define CLOCK_M_PKTGEN_ADDR_BASE 0x3F000
#define CLOCK_M_PKTGEN_ADDR_LEN 0x40000
#define CLOCK_M_PKTGEN_ADDR_RANGE_L (CLOCK_M_PKTGEN_ADDR_BASE)
#define CLOCK_M_PKTGEN_ADDR_RANGE_H (CLOCK_M_PKTGEN_ADDR_RANGE_L + CLOCK_M_PKTGEN_ADDR_LEN)


/// FIFO control register map (TSFIFO_CTRL)
enum TSFIFO_CTRL_bits {
	RXFEMPTY = 0x00000001,
	RXFFULL = 0x00000002,
	TXFEMPTY = 0x00010000,
	TXFFULL = 0x00020000
};

#define RXFCNT(value) (((value)&0x0000FFFC) >> 2)
#define TXFCNT(value) (((value)&0xFFFC0000) >> 18)

/// interrupt bits (in IRSRC and IREN)
enum IRQ_bits {
	IRQB_RECV_MAC = 0x00000001, ///< MAC received packet interrupt bit
	IRQB_TSFIFORX_FULL = 0x00000002, ///< time stamp FIFO full interrupt bit
	IRQB_SEND_MAC = 0x00000004, ///< MAC received packet interrupt bit
	IRQB_TSFIFOTX_FULL = 0x00000008, ///< time stamp FIFO full interrupt bit
	IRQB_EVENT0 = 0x00000400, ///< event 0 interrupt bit
	IRQB_EVENT1 = 0x00000800, ///< event 1 interrupt bit
	IRQB_TRIGGER0 = 0x00001000, ///< trigger 0 interrupt bit
	IRQB_TRIGGER1 = 0x00002000, ///< trigger 1 interrupt bit
	IRQB_PCLK = 0x00010000, ///< programmable clock interrupt bit
	IRQB_PERIOD0 = 0x01000000, ///< period time 0 interrupt bit
	IRQB_PERIOD1 = 0x02000000, ///< period time 1 interrupt bit
	IRQB_TIME_OFLW = 0x80000000 ///< time overflow interrupt bit
};

enum clock_m_irsc {
	CLOCK_M_IRSC_MIITS0RXTS,
	CLOCK_M_IRSC_TSFIFO0RXFULL,
	CLOCK_M_IRSC_MIITS0TXTS,
	CLOCK_M_IRSC_TSFIFO0TXFULL,
	CLOCK_M_IRSC_MIITS1RXTS,
	CLOCK_M_IRSC_TSFIFO1RXFULL,
	CLOCK_M_IRSC_MIITS1TXTS,
	CLOCK_M_IRSC_TSFIFO1TXFULL,
	CLOCK_M_IRSC_EVENT0OCCURED = 10,
	CLOCK_M_IRSC_EVENT1OCCURED,
	CLOCK_M_IRSC_TRIGGER0MATCH,
	CLOCK_M_IRSC_TRIGGER1MATCH,
	CLOCK_M_IRSC_PERIOD0ELAPSED = 24,
	CLOCK_M_IRSC_PERIOD1ELAPSED,
	CLOCK_M_IRSC_TIMEOVERFLOW = 31
};

/// event control bits (in EVENTCTRL)
enum EVENTCTRL_bits {
	EVENTCTRL_EVENT0_ENABLED = BIT(0), ///< event 0 control bit
	EVENTCTRL_EVENT1_ENABLED = BIT(1), ///< event 1 control bit
	EVENTCTRL_TRIGGER0_ENABLED = BIT(2), ///< trigger 0 control bit
	EVENTCTRL_TRIGGER1_ENABLED = BIT(3), ///< trigger 1 control bit
	EVENTCTRL_PERIOD0_ENABLED = BIT(4), ///< period 0 timer control bit
	EVENTCTRL_PERIOD1_ENABLED = BIT(5), ///< period 1 timer control bit
	EVENTCTRL_PERIOD0_OUTPUT_ENABLED = BIT(6),
	EVENTCTRL_PERIOD1_OUTPUT_ENABLED = BIT(7),
	EVENTCTRL_PERIOD0_INITIAL_VALUE = BIT(8),
	EVENTCTRL_PERIOD0_OUTPUT_VALUE = BIT(9),
	EVENTCTRL_PERIOD1_INITIAL_VALUE = BIT(10),
	EVENTCTRL_PERIOD1_OUTPUT_VALUE = BIT(11),
	EVENTCTRL_IRIGB_ENABLED = BIT(12),
	EVENTCTRL_PERIOD01_DUTY = BIT(13),
	EVENTCTRL_PERIOD0_START = BIT(15), ///< period timer control bits (16:15)
	EVENTCTRL_PERIOD1_START = BIT(17), ///< period timer control bits (18:17)
	EVENTCTRL_PERIOD2_ENABLED = BIT(19),
	EVENTCTRL_PERIOD3_ENABLED = BIT(20),
};

enum GENLOCK_CTRL_bits {
	GENLOCK_CTRL_PRECISION0ENBL = BIT(0),
	GENLOCK_CTRL_PRECISION1ENBL = BIT(1),
};

/// time cotnrol bits (in TIMECTRL)
enum TIMECTRL_bits {
	TIMECTL_LOADALL =
		BIT(0), ///< immediate load of all registers with shadow value
	TIMECTL_CROSSTIME =
		BIT(4), ///< Disable activation of timestamped packet flag in TX BD word
	TIMECTL_TIMEFORMAT =
		BIT(16), ///< Enable (1) NTP timestamp format for all timestampers. Otherwise PTP format (0).
	TIMECTL_STOPCLOPCK = BIT(28),
	TIMECTL_LOADSTEP =
		BIT(29), ///< load pure phase step size register with shadow value
	TIMECTL_ADDTIME =
		BIT(30), ///< add time in SHDWSTEPP_NS (interpreted as unsigned 30 bit) to current time
	TIMECTL_LOADTIME = BIT(31) ///< load time register with shadow value
};

/// Capabilities 1 bits
enum CAPABILITIES1_bits {
	CAP1_MIITS0 = 0x00000001, ///< MIITS0 implemented
	CAP1_MIITS1 = 0x00000002, ///< MIITS1 implemented
	CAP1_ULTIMATE = 0x00000004, ///< Ultimate timestamper implemented
	CAP1_MEASURE = 0x00000008, ///< event 16 for measurement implemented
	CAP1_EVENT0 = 0x00000010, ///< event 0 implemented
	CAP1_EVENT1 = 0x00000020, ///< event 1 implemented
	CAP1_TRIGGER0 = 0x00000040, ///< trigger 0 implemented
	CAP1_TRIGGER1 = 0x00000080, ///< trigger 1 implemented
	CAP1_PERIOD0 = 0x00000100, ///< period 0 implemented
	CAP1_PERIOD1 = 0x00000200, ///< period 1 implemented
	CAP1_TIME2 = 0x00000400, ///< time 2 registers implemented
	CAP1_IRIGB = 0x00000800, ///< IRIG-B output implemented
	CAP1_PROG1STEP = 0x00001000, ///< programmable one step implemented
	CAP1_PERIODSET =
		0x00002000, ///< PERIOD set and initial value is implemented
	CAP1_NTPMODE = 0x00004000, ///< NTP timestamp mode is implemented
	CAP1_LEAPAPPLY = 0x00008000 ///< leap apply time is implemented
};

/**
 * <!--********************************************************************************-->
 * \name Ethernet Media Access Controller registers
 * <!--********************************************************************************-->
 * @{
 */

/// Ethernet MAC registers
enum mac_registers {
	MAC_I2C_BASE = 0x000, ///< Base Address for i2c-ocore
	MAC_LINE_STATUS = 0x020,
	MAC_CAPABILITY_0 = 0x030, ///< capability reg 0 of mac
	MAC_CAPABILITY_1 = 0x034, ///< capability reg 1 of mac
	MAC_CAPABILITY_2 = 0x038, ///< capability reg 2 of mac
	MAC_CAPABILITY_3 = 0x03C, ///< capability reg 3 of mac
	MAC_ETH_VERSION = 0x040, ///< Version ID of the Ethernet MAC R
	MAC_ETH_BUILD = 0x044, ///< Build ID of the Ethernet MAC R
	MAC_MODE = 0x048, ///< MAC Mode Control Register R/W
	MAC_INT_SOURCE = 0x04C, ///< interrupt source R/W
	MAC_INT_MASK = 0x050, ///< interrupt mask R/W
	MAC_BURST_LENGTH = 0x054, ///< Burst Length Register R/W
	MAC_BD_NUM =
		0x058, ///< number of receive and transmit buffer descriptors R/W
	MAC_TXPAUSE = 0x05C, ///< transmit pause control R/W
	MAC_ETH_ADDR0 = 0x060, ///< lower part of the Ethernet MAC address R/W
	MAC_ETH_ADDR1 = 0x064, ///< upper part of the Ethernet MAC address R/W
	MAC_MIIMODE = 0x068, ///< MII mode register R/W
	MAC_MIICOMMAND = 0x06C, ///< MII command select register R/W
	MAC_MIIADDRESS = 0x070, ///< MII register selection R/W
	MAC_MIITX_DATA = 0x074, ///< MII transmit data R/W
	MAC_MIIRX_DATA = 0x078, ///< MII receive data and status register R
	MAC_IFG_CTRL = 0x07C, ///< Interframe Gap Control R/W
	MAC_ETH_HASH0_ADR =
		0x080, ///< lower Ethernet multicast hash comparison register R/W
	MAC_ETH_HASH1_ADR =
		0x084, ///< upper Ethernet multicast hash comparison register R/W
	MAC_VLAN0 = 0x088, ///< virtual LAN VID register 0 R/W
	MAC_VLAN1 = 0x08C, ///< virtual LAN VID register 1 R/W
	MAC_VLAN2 = 0x090, ///< virtual LAN VID register 2 R/W
	MAC_VLAN3 = 0x094, ///< virtual LAN VID register 3 R/W
	MAC_TS_RXCTRL =
		0x098, ///< Generic receive time stamper control register
	MAC_TS_TXCTRL =
		0x09C, ///< Generic transmit time stamper control register
	MAC_CNT_RXERR = 0x0A0, ///< erroneous receive packets counter R/W
	MAC_CNT_FCSERR = 0x0A4, ///< packets with failed FCS check R/W
	MAC_CNT_MCASTERR = 0x0A8, ///< packets with multicast errors R/W
	MAC_CNT_LCOLLERR = 0x0AC, ///< number of late collisions R/W
	MAC_CNT_LENERR = 0x0B0, ///< packet length errors R/W
	MAC_CNT_OVERERR = 0x0B4, ///< receive path overrun errors R/W
	MAC_CNT_FRAMEERR = 0x0B8, ///< packets with frame length errors R/W
	MAC_CNT_RX = 0x0BC, ///< Statistic counter: receive packets R
	MAC_CNT_TX = 0x0C0, ///< Statistic counter: transmit packets R
	MAC_CNT_TXERR = 0x0C4, ///< Statistic counter: erroneous receive packets
	MAC_CNT_PTP_TXTS = 0x0C8,
	MAC_CNT_PTP_RXTS = 0x0CC,
	MAC_TS_PHY_DLY_TX = 0x0D0, ///< PHY delay data transmit R/W
	MAC_TS_PHY_DLY_RX = 0x0D4, ///< PHY delay data receive	R/W
	MAC_TS_CTRL = 0x0D8, ///< Timestamper control
	MAC_1STEPCTRL = 0x0DC, ///< one step control R/W
	MAC_VLAN_GRP_ADR = 0x0E0, ///< VLAN VID group register
	MAC_CNT_VLAN_ERR = 0x0E4, ///< VLAN missmatch counter
	MAC_CNT_VLAN = 0x0E8, ///< VLAN accept counter
	MAC_CNT_BROADCAST = 0x0EC, ///< broadcast message counter
	MAC_CNT_MULTICAST = 0x0F0, ///< multicast message counter
	MAC_CNT_RX_TS = 0x0F4, ///< received and timestamped message counter
	MAC_CNT_TX_TS = 0x0F8, ///< transmitted and timestamped message counter
	MAC_CNT_RX_TS2 =
		0x0FC, ///< received and timestamped message counter for 2nd MAC TS
	MAC_CNT_TX_TS2 =
		0x100, ///< transmitted and timestamped message counter for 2nd MAC TS
	MAC_TS2_1STEPCTRL = 0x104, ///< one step control for 2nd MAC TS R/W
	MAC_TS2_RXCTRL =
		0x108, ///< Generic receive time stamper control register for 2nd MAC TS
	MAC_TS2_TXCTRL =
		0x10C, ///< Generic transmit time stamper control register for 2nd MAC TS
	MAC_PTP_TS_CTRL = 0x110,
	MAC_BD_BASE = 0x400, ///< buffer descriptor base address R/W
	MAC_TS_RXMEM_BASE =
		0xC00, ///< Generic receive time stamper mask/pattern memory W
	MAC_TS2_RXMEM_BASE =
		0xD00, ///< Generic receive time stamper mask/pattern memory for 2nd MAC TS W
	MAC_TS_TXMEM_BASE =
		0xE00, ///< Generic transmit time stamper mask/pattern memory W
	MAC_TS2_TXMEM_BASE =
		0xF00 ///< Generic transmit time stamper mask/pattern memory for 2nd MAC TS W
};

enum MAC_LINE_STATUS {
	MAC_LINE_STATUS_LINK = BIT(0),
	MAC_LINE_STATUS_PRESENT = BIT(1),
};

enum MAC_PTP_TS_CTRL {
	MAC_PTP_TS_CTRL_ENABLE_TX = BIT(0),
	MAC_PTP_TS_CTRL_ENABLE_TX_1STEP = BIT(1),
	MAC_PTP_TS_CTRL_ENABLE_RX = BIT(7),
};

/// maximum number of buffer descriptors
#define MAC_TOTAL_BD 128

/// required Syn1588(R) CLOCK_M version (Major, Minor) for these register definitions
#define REQ_MAC_PROG1TS_VER \
	0x33313430 // Programmable one step timestamper, Build 3.1.40
#define REQ_MAC_TS_VER \
	0x33313330 // Mac with integrated timestamper, Build 3.1.30
#define REQ_MAC_VER 0x3331 // Build 3.1.xx /*0x3330*/
#define REQ_MAC_VER2 0x3332 // Build 3.2.xx
#define REQ_MAC_OLD_VER 0x3330 // Build 3.0.xx
#define REQ_XMAC_VER 0x3431 // Build 4.1.xx for 10G MAC

/// TS mask/pattern memory size
#define MAC_TS_MEM_SIZE 0x100

/// Transmit Buffer Descriptor Flags
enum MAC_TX_BD_flags {
	MAC_TX_BD_READY = 0x8000, ///< Tx BD Ready
	MAC_TX_BD_IRQ = 0x0400, ///< Tx BD IRQ Enable
	//MAC_TX_BD_WRAP			 = 0x2000, ///< Tx BD Wrap (last BD)
	//MAC_TX_BD_PAD				= 0x1000, ///< Tx BD Pad Enable FIXME! remove n/a in HW
	//MAC_TX_BD_CRC				= 0x0800, ///< Tx BD CRC Enable FIXME! remove n/a in HW
	MAC_TX_BD_TIMESTAMP = 0x0100, ///< Tx BD Timestamp attached
	MAC_TX_BD_RETRY = 0x00F0, ///< Tx BD Retry Status
	MAC_TX_BD_RETLIM = 0x0008, ///< Tx BD Retransmission Limit Status
	MAC_TX_BD_LATECOL = 0x0004, ///< Tx BD Late Collision Status
	MAC_TX_BD_DEFER = 0x0002, ///< Tx BD Defer Status
	MAC_TX_BD_CARRIER = 0x0001, ///< Tx BD Carrier Sense Lost Status
	MAC_TX_BD_STATS =
		(MAC_TX_BD_RETRY | MAC_TX_BD_RETLIM | MAC_TX_BD_LATECOL |
		 MAC_TX_BD_DEFER | MAC_TX_BD_CARRIER)
};

/// Receive Buffer Descriptor Flags
enum MAC_RX_BD_flags {
	MAC_RX_BD_EMPTY = 0x8000, ///< Rx BD Empty
	MAC_RX_BD_IRQ = 0x0400, ///< Rx BD IRQ Enable
	MAC_RX_BD_VLANFRAME = 0x0200, ///< Rx BD VLAN Frame
	MAC_RX_BD_CTRLFRAME = 0x0100, ///< Rx BD Control Frame
	MAC_RX_BD_MISS = 0x0080, ///< Rx BD Miss Status
	MAC_RX_BD_TIMESTAMP = 0x0040, ///< Rx BD Timestamp attached
	MAC_RX_BD_INVSYMB = 0x0020, ///< Rx BD Invalid Symbol Status
	MAC_RX_BD_DRIBBLE = 0x0010, ///< Rx BD Dribble Nibble Status
	//MAC_RX_BD_TOOLONG		= 0x0008, ///< Rx BD Too Long Status
	MAC_RX_BD_SHORT = 0x0004, ///< Rx BD Too Short Frame Status
	MAC_RX_BD_CRCERR = 0x0002, ///< Rx BD CRC Error Status
	MAC_RX_BD_LATECOL = 0x0001, ///< Rx BD Late Collision Status
	MAC_RX_BD_STATS =
		(MAC_RX_BD_VLANFRAME | MAC_RX_BD_CTRLFRAME | MAC_RX_BD_MISS |
		 MAC_RX_BD_TIMESTAMP | MAC_RX_BD_INVSYMB | MAC_RX_BD_DRIBBLE |
		 MAC_RX_BD_SHORT | MAC_RX_BD_CRCERR | MAC_RX_BD_LATECOL)
};

/// Mode register flags
enum MAC_MODE_flags {
	MAC_MODE_RXEN = 0x00000001, ///< Receive Enable
	MAC_MODE_TXEN = 0x00000002, ///< Transmit Enable
	MAC_MODE_NOPRE = 0x00000004, ///< No Preamble
	MAC_MODE_BRO = 0x00000008, ///< Reject Broadcast
	MAC_MODE_SWRST = 0x00000010, ///< Software Reset
	MAC_MODE_FULLD = 0x00000020, ///< Full Duplex
	MAC_MODE_GBIT = 0x00000040, ///< Gigabit Ethernet enable for MAC
	MAC_MODE_LOOPBCK = 0x00000080, ///< Loop Back
	MAC_MODE_CRCEN = 0x00000100, ///< CRC Enable
	MAC_MODE_EXDFREN = 0x00000200, ///< Excess Defer
	MAC_MODE_NOBCKOF = 0x00000400, ///< No Backoff
	MAC_MODE_PAD = 0x00000800, ///< Pad Enable
	MAC_MODE_DLYCRCEN = 0x00001000, ///< Delayed CRC enable
	MAC_MODE_HUGEN = 0x00002000, ///< Huge enable
	MAC_MODE_RECSMALL = 0x00004000, ///< receive Small
	MAC_MODE_SUPPRESS_FCS = 0x00008000, ///< Suppress forwarding of FCS by RX MAC
	MAC_MODE_PRO = 0x00010000, ///< promiscuous (receive all)
	MAC_MODE_IFG = 0x00020000, ///< min. IFG not required
	MAC_MODE_VLANEN = 0x00040000, ///< VLAN filtering enable
	MAC_MODE_PASSCTRL = 0x00080000, ///< pass received control frames
	MAC_MODE_RXPAUSE = 0x00100000, ///< receive pause control frames
	MAC_MODE_TXPAUSE = 0x00200000, ///< transmit pause enable
	MAC_MODE_1000BASE_T = BIT(22),
	MAC_MODE_1000BASE_X = BIT(23),
};

/// Interrupt Source Register, Interrupt Mask Register Flags
enum MAC_INT_flags {
	MAC_INT_TXB = 0x00000001, ///< Transmit Buffer IRQ
	MAC_INT_TXE = 0x00000002, ///< Transmit Error IRQ
	MAC_INT_TXC = 0x00000004, ///< Transmit Control Frame IRQ
	MAC_INT_RXF = 0x00000008, ///< Receive Frame IRQ
	MAC_INT_RXE = 0x00000010, ///< Receive Error IRQ
	MAC_INT_RXC = 0x00000020, ///< Received Control Frame IRQ
	MAC_INT_BUSY = 0x00000040, ///< Busy IRQ
	MAC_INT_OVER = 0x00000080 ///< FIFO overrun IRQ
};

/// Capabilities 0 bits
enum MAC_CAPABILITIES_0_bits {
	MAC_CAP0_RXTS         =    1 << 0,  ///< RX TS unit implemented
	MAC_CAP0_TXTS         =    1 << 1,  ///< TX TS unit implemented
	MAC_CAP0_1STEP        =    1 << 2,  ///< TX 1-step TS unit implemented
	MAC_CAP0_STAT_CNT_LEN =    1 << 3,  ///< length of statistic counters: 0..16 bit, 1..32 bit
	MAC_CAP0_VLAN_ID_NUM  =    1 << 4,  ///< number of VLAN IDs: 0..1 VLAN, 1..4 VLANs
	MAC_CAP0_VLAN_GRP     =    1 << 5,  ///<  VLAN group is implemented
	MAC_CAP0_EXT_STAT     =    1 << 6,  ///<  extended statistic counters implemented
	MAC_CAP0_SFP_ID       =    1 << 7,  ///<  SFP transceiver module identification implemented in HW (useless)
	MAC_CAP0_10G_MAC      =    1 << 8,  ///<  unit is 10Gbit MAC
	MAC_CAP0_1G_MAC       =    1 << 9,  ///<  unit is 100/1000Mbit MAC
	MAC_CAP0_25G_MAC      =    1 << 10, ///<  unit is 25Gbit MAC
					    ///< reserved
	MAC_CAP0_FEC          =    1 << 11, ///<  FEC function available
	MAC_CAP0_JUMBO        =    1 << 13, ///<  Jumbo packet support implemented
	MAC_CAP0_AXI          =    1 << 14, ///<  AXI interface with variable burst length implemented
	MAC_CAP0_TS2          =    1 << 15, ///<  second timestamping units implemented
	MAC_CAP0_FIBER        =    1 << 16, ///<  Fiber interface status implemented (only for 10/25 Gbit MAC)
	MAC_CAP0_SECURITY     =    1 << 17, ///<  PTP V2.1 security enabled
	MAC_CAP0_VARIABLE_BURST=   1 << 18,///<  Variable burst length implemented
	MAC_CAP0_SW_MDIO      =    1 << 19, ///<
	MAC_CAP0_I2C          =    1 << 20, ///<
	MAC_CAP0_PTP_TS       =    1 << 21, ///<
};
/// Capabilities 1 bits
enum MAC_CAPABILITIES_1_bits {
	MAC_CAP1_INTERFRAME_GAP  = 1 << 0,  ///< Programmable inter-frame gap implemented
};

/**
 * \name General Register Bit Flags
 * @{
 */
/* TXPAUSE Register */
#define MAC_TXPAUSE_REQ 0x00010000 ///< Send pause frame

/* MII Mode Register */
#define MAC_MIIMODE_CLKDIV 0x000000FF ///< Clock Divider
#define MAC_MIIMODE_NOPRE 0x00000100 ///< No Preamble

/* MII Command Register	*/
// #define MAC_MIICOMMAND_SCANSTAT 0x00000001 ///< Scan Status VOIP only
#define MAC_MIICOMMAND_READ  (1u << 1) ///< Read Status
#define MAC_MIICOMMAND_WRITE (1u << 2) ///< Write Control Data
#define MAC_MIICOMMAND_PHY_DETECTED (1u << 3) ///< PHY detected
#define MAC_MIICOMMAND_SFP_PORT (1u << 4) ///< interface has a SFP Port
#define MAC_MIICOMMAND_SFP_MODULE_DETECTED (1u << 5) ///< SFP module detected on Port
#define MAC_MIICOMMAND_SFP_BASEX (1u << 6) ///<  SFP module is a basex
#define MAC_MIICOMMAND_SFP_BASET (1u << 7) ///<  SFP module is a baset

/* MII Address Register */
#define MAC_MIIADDRESS_FIAD 0x0000001F ///< PHY address
#define MAC_MIIADDRESS_RGAD 0x00001F00 ///< RGAD address

/* MII Status in MIIRX_DATA Register */
//#define MAC_MIISTATUS_LINKFAIL 0x00010000 ///< Link Fail
#define MAC_MIISTATUS_BUSY 0x00020000 ///< MII Busy
#define MAC_MIISTATUS_NVALID 0x00040000 ///< MII Status is invalid

/* ETH_ADDR1 Register */
#define MAC_ETH_ADDR1_SW 0x00010000 ///< override HW MAC address bit

/* VLAN Register */
#define MAC_VLAN_VLANEN 0x00010000 ///< enable VLAN ID for filter

/*}*/

/* Ethernet buffer descriptor */
/*	struct mac_bd {
		ushort	len;						// Buffer length
		ushort	status;				 // Buffer status
		uint		addr;					 // Buffer address
	};*/

// TODO: complete list and write documentation
/// register set for ETHTOOL register dump output
#define MAC_REGSET													\
	{																\
		MAC_ETH_VERSION, MAC_ETH_BUILD, MAC_MODE, MAC_INT_SOURCE,	\
			MAC_INT_MASK, MAC_BURST_LENGTH, MAC_BD_NUM,				\
			MAC_TXPAUSE, MAC_ETH_ADDR0, MAC_ETH_ADDR1,				\
			MAC_MIIMODE, MAC_MIICOMMAND, MAC_MIIADDRESS,			\
			MAC_MIITX_DATA, MAC_MIIRX_DATA, MAC_IFG_CTRL,			\
			MAC_ETH_HASH0_ADR, MAC_ETH_HASH1_ADR, MAC_VLAN0,		\
			MAC_VLAN1, MAC_VLAN2, MAC_VLAN3							\
	}


#endif /* _CHAR_SYN1588NIC_REG_ */
