
#include "clock_m.h"
#include "oregano_tse_mac.h"
#include "syn1588nic_local.h"
#include "syn1588_debugfs.h"

#include <linux/debugfs.h>
#include <linux/miscdevice.h>



void syn1588_debugfs_init_bd(struct syn1588_adapter *adapter)
{
	struct nic_priv_data *nic_dat = netdev_priv(adapter->nic);
	struct dentry *root_dir = adapter->debugfs->root_dir;
	struct dentry *buffer_descriptor_dir;
	char name_rx_bd[] = "rx_bd_xxx";
	char name_tx_bd[] = "tx_bd_xxx";
	char name_tx_bd_time[] = "tx_bd_time_xxx";
	char name_tx_bd_time_ns[] = "tx_bd_time_ns_xxx";
	u32 *mac_base = (u32 *)nic_dat->iobase;

	// cast memory to non volatile as it is not supported by the api
	u32 *mac_tx_base = (u32 *)mac_base + (MAC_BD_BASE >> 2);
	u32 *mac_rx_base = (u32 *)mac_base + ((MAC_BD_BASE + 64*16) >> 2);

	int i; // counter variable

	buffer_descriptor_dir = debugfs_create_dir("buffer_descriptor", root_dir);
	if (IS_ERR_OR_NULL(buffer_descriptor_dir)) {
		pr_err("can't create debugfs dir\n");
		//return -1;
		return;
	}

	debugfs_create_u8("rx_rp", 0440, buffer_descriptor_dir, &nic_dat->rx_dma_bd_rp);
	debugfs_create_u8("tx_rp", 0440, buffer_descriptor_dir, &nic_dat->tx_dma_bd_rp);
	debugfs_create_u8("tx_wp", 0440, buffer_descriptor_dir, &nic_dat->tx_dma_bd_wp);

	for (i = 0; i < 64; i++) {
		snprintf(name_rx_bd + 6, 3, "%d", i);
		snprintf(name_tx_bd + 6, 3, "%d", i);
		snprintf(name_tx_bd_time + 6 + 5, 3, "%d", i);
		snprintf(name_tx_bd_time_ns + 6 + 5 + 3, 3, "%d", i);

		debugfs_create_x32(name_tx_bd, 0640, buffer_descriptor_dir,  &mac_tx_base[i * 16u/4]);
		debugfs_create_u32(name_tx_bd_time, 0640, buffer_descriptor_dir, &mac_tx_base[(i * 16 + 8) / 4]);
		debugfs_create_u32(name_tx_bd_time_ns, 0640, buffer_descriptor_dir, &mac_tx_base[(i * 16 + 12) / 4]);

		debugfs_create_x32(name_rx_bd, 0640, buffer_descriptor_dir, &mac_rx_base[(i * 16) / 4]);
	}
}

int eth_version_seq_show(struct seq_file *m, void *p)
{
	u8 version[5] = "";
	u32 eth_version_reg;
	struct syn1588_adapter *adapter = m->private;
	struct nic_priv_data *nic_dat = netdev_priv(adapter->nic);

	if (adapter == NULL) {
		pr_err("adapter is null");
		return 0;
	}
	if (adapter->iobase == NULL) {
		pr_err("iobase is null");
		return 0;
	}
	eth_version_reg = nic_readl(nic_dat, MAC_ETH_VERSION);

	version[0] = (eth_version_reg >> 24) & 0xff;
	version[1] = (eth_version_reg >> 16) & 0xff;
	version[2] = (eth_version_reg >> 8) & 0xff;
	version[3] = eth_version_reg & 0xff;
	version[4] = 0x00;

	seq_printf(m, "%s\n", version);
	return 0;
}

int eth_version_open_fn(struct inode *inode, struct file *file)
{
	return single_open(file, eth_version_seq_show, inode->i_private);
}

struct file_operations eth_version_file_ops = {
	.open = eth_version_open_fn,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = single_release,
	.owner = THIS_MODULE,
};

void syn1588_debugfs_init_syn1588_version(struct syn1588_adapter *adapter)
{
	struct dentry *root_dir = adapter->debugfs->root_dir;
	// cast memory to non volatile as it is not supported by the api
	struct nic_priv_data *nic_dat = netdev_priv(adapter->nic);
	u32 *mac_base = (u32 *)nic_dat->iobase;

	//debugfs_create_u32("eth_version", 0444, root_dir, &mac_base[MAC_ETH_VERSION/4]);
	debugfs_create_u32("eth_build", 0444, root_dir, &mac_base[MAC_ETH_BUILD/4]);

	debugfs_create_file("eth_version", 0444, root_dir, adapter, &eth_version_file_ops);
}


int clock_m_id_seq_show(struct seq_file *m, void *p)
{
	struct clock_m_device *clock_m = m->private;
	u32 data[2];

	if (clock_m == NULL) {
		pr_err("clock is null");
		return 0;
	}
	if (clock_m->iobase == NULL) {
		pr_err("iobase is null");
		return 0;
	}

	data[0] = clock_m_readl(clock_m, CLOCKID_L);
	data[1] = clock_m_readl(clock_m, CLOCKID_H);

	seq_printf(m, "%04x%08x\n", data[1], data[0]);
	return 0;
}

int clock_m_id_open_fn(struct inode *inode, struct file *file)
{
	return single_open(file, clock_m_id_seq_show, inode->i_private);
}

struct file_operations clock_m_id_file_ops = {
	.open = clock_m_id_open_fn,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = single_release,
	.owner = THIS_MODULE,
};

void syn1588_debugfs_init_clock_m(struct syn1588_adapter *adapter)
{

	struct dentry *root_dir = adapter->debugfs->root_dir;

	debugfs_create_file("clock_id", 0444, root_dir, adapter->clock_m, &clock_m_id_file_ops);

}

void syn1588_debugfs_init(struct syn1588_adapter *adapter)
{
	adapter->debugfs = kzalloc(sizeof(struct syn1588_debugfs), GFP_KERNEL);
	if (IS_ERR_OR_NULL(adapter->debugfs)) {
		pr_err("can't alloc space for debugfs\n");
		return;
	}
	if(!IS_ERR_OR_NULL(adapter->clock_m)) {
		adapter->debugfs->root_dir = debugfs_create_dir(adapter->clock_m->miscdev->name, NULL);
	} else {
		adapter->debugfs->root_dir = debugfs_create_dir(adapter->nic->name, NULL);
	}
	if (IS_ERR_OR_NULL(adapter->debugfs->root_dir)) {
		pr_err("can't create debugfs dir\n");
		syn1588_debugfs_exit(adapter);
		return;
	}
	if(adapter->nic) {
		syn1588_debugfs_init_bd(adapter);
		syn1588_debugfs_init_syn1588_version(adapter);
	}
	if(adapter->clock_m) {
		syn1588_debugfs_init_clock_m(adapter);
	}

}

void syn1588_debugfs_exit(struct syn1588_adapter *adapter)
{
	if (IS_ERR_OR_NULL(adapter->debugfs)) {
		// nothing to do here
		return;
	}
	if (!IS_ERR_OR_NULL(adapter->debugfs->root_dir)) {
		debugfs_remove_recursive(adapter->debugfs->root_dir);
	}


	kfree(adapter->debugfs);
	adapter->debugfs = NULL;
}
