



#ifndef SYN1588_DEV_H
#define SYN1588_DEV_H

#include <linux/device.h>
#include <linux/platform_device.h>
#include <linux/types.h>

struct clock_m_device;
struct net_device;
struct syn1588_product;


enum syn1588_product_id {
	SYN1588_PRODUCT_ID_NIC_REV2_0,
	SYN1588_PRODUCT_ID_NIC_REV2_1,
	SYN1588_PRODUCT_ID_NIC_REV2_3_A,
	SYN1588_PRODUCT_ID_NIC_REV2_3_B,
	SYN1588_PRODUCT_ID_NIC_REV2_TESTING,
	SYN1588_PRODUCT_ID_NIC_REV3,
	SYN1588_PRODUCT_ID_DUALNIC_REV1_0,
	SYN1588_PRODUCT_ID_DUALNIC_REV1_1,
};

#define MAX_AMOUNT_MACS 2

#define SERIAL_NR_MAX_LENGTH	16

/**
 * represent a syn1588nic
 *
 */
struct syn1588_dev {

	struct platform_device *clock_m_dev;
	struct platform_device *mac_devs[MAX_AMOUNT_MACS];
	size_t mac_dev_num;
	struct platform_device *sync_d;
	struct platform_device *updater;
	enum syn1588_product_id product_id; ///< unique product name or NULL

	// properties
	u32 build_id;
	char serial_nr[SERIAL_NR_MAX_LENGTH+1];
};

static inline const char * syn1588_dev_get_product_id(enum syn1588_product_id product_id) {
	switch(product_id) {
	case SYN1588_PRODUCT_ID_NIC_REV2_0:
		return "syn1588nic_rev2_0";
		break;
	case SYN1588_PRODUCT_ID_NIC_REV2_1:
		return "syn1588nic_rev2_1";
		break;
	case SYN1588_PRODUCT_ID_NIC_REV2_3_A:
		return "syn1588nic_rev2_3_a";
		break;
	case SYN1588_PRODUCT_ID_NIC_REV2_3_B:
		return "syn1588nic_rev2_3_b";
		break;
	case SYN1588_PRODUCT_ID_NIC_REV2_TESTING:
		return "syn1588nic_rev2_testing";
		break;
	case SYN1588_PRODUCT_ID_NIC_REV3:
		return "syn1588nic_rev3";
		break;
	case SYN1588_PRODUCT_ID_DUALNIC_REV1_0:
		return "syn1588dualnic_rev1_0";
		break;
	case SYN1588_PRODUCT_ID_DUALNIC_REV1_1:
		return "syn1588dualnic_rev1_1";
		break;
	default:
		return "unknown";
	}
}

#endif // _SYN1588NIC_LOCAL_H_
